###########################################
# The TopMon Superkaramba Theme           #
# Written by Stefaan Lippens, 2004-2005   #
# Released under GPL                      #
###########################################

#this import statement allows access to the karamba functions
import karamba, sys
import time

# interval (in seconds)  between "top" queries,
# this is unrelated to the refreshrate of the karamba theme
# You should not set this interval to small
tableRefreshDelay = 10


#settings (can be modified if you know what you are doing)
tableWidth = 100
tableHeight = 80
numberOfLines = 6
fontSize = 9
tablePadding = { "top":6, "right":5, "bottom":6, "left":10 }

#further globals
memoryUsageTableTextFields =[]
cpuUsageTableTextFields =[]
tableAvailableHeight = tableHeight - tablePadding["top"] - tablePadding["bottom"]
tableAvailableWidth = tableWidth - tablePadding["right"] - tablePadding["left"]
lineHeight = int(fontSize*1.2)
linePadding = int((tableAvailableHeight - numberOfLines * lineHeight) / (numberOfLines+1))
lineSpacing = lineHeight + linePadding
lineOffset = tablePadding["top"]
col1Offset = 70 + tablePadding["left"]
col2Width = int(3*0.8*fontSize)
col2Offset = 60 + int(tableWidth - tablePadding["right"] - col2Width)
col1Width = tableAvailableWidth - col2Width - 10
memoryPaneOffset = 85
cpuPaneOffset = 0
ignoretop = 1
lastTime = time.time() - tableRefreshDelay


#the class for extracting the memory usage out of top's output
class CpuAndMemoryUsage:
    def __init__(self,widget):
        self.memtable=[]
        self.cputable=[]
        import os
        #open pipe with "top" and read output
        os.environ["TERM"] = "xterm"
        (toppipein, toppipe) = os.popen4( "top -b -n1", "r")
        toppipein.close()
        toplist = toppipe.readlines()
        toppipe.close()
        #get interesting column indexes
        headerLine = toplist[6]
        header = headerLine.split()
        try:
            memFracColIndex = header.index("%MEM")
            cpuFracColIndex = header.index("%CPU")
            comandColIndex  = header.index("COMMAND")
        except ValueError:
            print "ValueError exception caught"
            for line in toplist:
                print line
            print headerLine
            print header
            raise
        #build tables
        for line in toplist[7:]:
            entry = line.split()
            if len(entry)<1:
                continue
            memFrac = float( entry[memFracColIndex])
            cpuFrac = float( entry[cpuFracColIndex])
            cmdname = entry[comandColIndex]
            if ignoretop and cmdname != "top":
                self.addentry(cmdname, cpuFrac, memFrac)
        #sort on memFrac
        self.sort()
    def addentry(self, cmdname, cpuFrac, memFrac):
        for entry in self.cputable:
            if cmdname == entry["c"]:
                entry["n"] += 1
                entry["f"] += cpuFrac
                break
        else:
            self.cputable.append({"c":cmdname,"n":1,"f":cpuFrac})
        for entry in self.memtable:
            if cmdname == entry["c"]:
                entry["n"] += 1
                entry["f"] += memFrac
                break
        else:
            self.memtable.append({"c":cmdname,"n":1,"f":memFrac})
    def sort(self):
        self.memtable.sort(lambda x,y: -cmp(x["f"],y["f"]))
        self.cputable.sort(lambda x,y: -cmp(x["f"],y["f"]))



#this is called when you widget is initialized
def initWidget(widget):
    global numberOfLines, fontSize, titleFontSize, lineHeight, lineOffset, col1Offset , col2Offset, lineSpacing, col2Width, tablePadding, tableWidth, tableHeight
    global debugFile
    debugFileName = karamba.getThemePath(widget) + "/.topmon.debug"
    debugFile = open(debugFileName,"w")
    sys.stdout = debugFile
    sys.stderr = debugFile
    debugFile.write("#this file is only for debugging purposes,\n#deleting it is no problem\n")
    debugFile.flush()
    #create textfields
    h = int(fontSize*1.5)
    #create memtable textfields
    y =  cpuPaneOffset + lineOffset
    global cpuUsageTableTextFields
    for c in range(numberOfLines):
        k = karamba.createText(widget, col1Offset, y, col1Width , h, "? ")
        karamba.changeTextColor(widget, k, 0, 0, 0)
        karamba.changeTextSize(widget, k, fontSize)
        v = karamba.createText(widget, col2Offset, y, col2Width, h,"?%")
        karamba.changeTextColor(widget, v, 0, 0, 0)
        karamba.changeTextSize(widget, v, fontSize)
        cpuUsageTableTextFields.append((k,v))
        y += lineSpacing
    #create memtable textfields
    y =  memoryPaneOffset + lineOffset
    global memoryUsageTableTextFields
    for c in range(numberOfLines):
        k = karamba.createText(widget, col1Offset, y, col1Width , h, "? ")
        karamba.changeTextColor(widget, k, 0, 0, 0)
        karamba.changeTextSize(widget, k, fontSize)
        v = karamba.createText(widget, col2Offset, y, col2Width, h,"?%")
        karamba.changeTextColor(widget, v, 0, 0, 0)
        karamba.changeTextSize(widget, v, fontSize)
        memoryUsageTableTextFields.append((k,v))
        y += lineSpacing
    widgetUpdated(widget)

#this is called everytime your widget is updated
#the update inverval is specified in the .theme file
def widgetUpdated(widget):
    global memoryUsageTableTextFields
    global cpuUsageTableTextFields
    global lastTime, tableRefreshDelay
    now = time.time()
    if (now > lastTime + tableRefreshDelay ):
        cpuAndMemoryUsage = CpuAndMemoryUsage(widget)
        for c in range(numberOfLines):
            #cpu
            name = cpuAndMemoryUsage.cputable[c]["c"] + " "
            if cpuAndMemoryUsage.cputable[c]["n"] > 1:
                name += "(x%d)" % cpuAndMemoryUsage.cputable[c]["n"]
            karamba.changeText(widget, cpuUsageTableTextFields[c][0], name)
            karamba.changeText(widget, cpuUsageTableTextFields[c][1], "%d%%" % int(cpuAndMemoryUsage.cputable[c]["f"]))
            #mem
            name = cpuAndMemoryUsage.memtable[c]["c"] + " "
            if cpuAndMemoryUsage.memtable[c]["n"] > 1:
                name += "(x%d)" % cpuAndMemoryUsage.memtable[c]["n"]
            karamba.changeText(widget, memoryUsageTableTextFields[c][0], name)
            karamba.changeText(widget, memoryUsageTableTextFields[c][1], "%d%%" % int(cpuAndMemoryUsage.memtable[c]["f"]))
        lastTime = time.time()



#This gets called everytime our widget is clicked.
#Notes:
#  widget = reference to our widget
#  x = x position (relative to our widget)
#  y = y position (relative to our widget)
#  botton = button clicked:
#                    1 = Left Mouse Button
#                    2 = Middle Mouse Button
#                    3 = Right Mouse Button, but this will never happen
#                        because the right mouse button brings up the
#                        Karamba menu.
#                    4,5 = Scroll wheel up and down
def widgetClicked(widget, x, y, button):
    pass

#This gets called everytime our widget is clicked.
#Notes
#  widget = reference to our widget
#  x = x position (relative to our widget)
#  y = y position (relative to our widget)
#  botton = button being held:
#                    0 = No Mouse Button
#                    1 = Left Mouse Button
#                    2 = Middle Mouse Button
#                    3 = Right Mouse Button, but this will never happen
#                        because the right mouse button brings up the
#                        Karamba menu.
def widgetMouseMoved(widget, x, y, button):
    #Warning:  Don't do anything too intensive here
    #You don't want to run some complex piece of code everytime the mouse moves
    pass


#This gets called when an item is clicked in a popup menu you have created.
#  menu = a reference to the menu
#  id = the number of the item that was clicked.
def menuItemClicked(widget, menu, id):
    pass

#This gets called when an item is clicked in the theme CONFIGURATION menu,
#not the popup menus that you create.
#  key = the reference to the configuration key that was changed
#  value = the new value (true or false) that was selected
def menuOptionChanged(widget, key, value):
    pass

#This gets called when a meter (image, text, etc) is clicked.
# NOTE you must use attachClickArea() to make a meter
# clickable.  
#  widget = reference to your theme
#  meter = the meter clicked
#  button = the button clicked (see widgetClicked for button numbers)
def meterClicked(widget, meter, button):
    pass

#This gets called when a command you have executed with executeInteractive() outputs something
#to stdout.  This way you can get the output of for example kdialog without freezing up the widget
#waiting for kdialog to end.
#  widget = reference to your theme
#  pid = process number of the program outputting (use this if you execute more than out process)
#  output = the text the program outputted to stdout
def commandOutput(widget, pid, output):
    pass

#This gets called when an item is dropped on this widget.
# NOTE you have to call acceptDrops() before your widget will accept drops.
#  widget = reference to your theme
#  dropText = the text of the dropped item (probably a URL to it's location in KDE)
def itemDropped(widget, dropText):
    pass


#This gets called when a new program is LOADING in KDE.  When it is done
#loading, startupRemoved() is called, followed by taskAdded().
#  widget = reference to your widget
#  task = A refence to the task that is starting.  
def startupAdded(widget, startup):
    pass

#This gets called when a new program is done LOADING in KDE.
#  widget = reference to your widget
#  task = A refence to the task that just finished loading.  
def startupRemoved(widget, startup):
    pass

#This is called every time a new task (program) is started in KDE.
#  widget = reference to your widget
#  task = A refence to the new task.  Call getTaskInfo() with this reference
#         to get the name, etc of this new task.
def taskAdded(widget, task):
    pass

#This is called everytime a task (program) is closed in KDE.
#  widget = reference to your widget
#  task = A refence to the task.  
def taskRemoved(widget, task):
    pass

#This is called everytime a different task gains focus (IE, the user clicks
#on a different window).  
#  widget = reference to your widget
#  task = A refence to the task.  Call getTaskInfo() with this reference
#         to get the name, etc of this new task.
def activeTaskChanged(widget, task):
    pass

# This will be printed when the widget loads.
print "Loaded topmon!"

